import _extends from '@babel/runtime/helpers/esm/extends';
import React, { useRef, useEffect } from 'react';
import PropTypes from 'prop-types';
import { useSprings, animated } from 'react-spring';
import { useDrag } from 'react-use-gesture';
import useResizeObserver from 'use-resize-observer';
import 'intersection-observer';

const slidesWrapperStyle = vertical => ({
  display: 'flex',
  height: '100%',
  width: '100%',
  flexWrap: 'nowrap',
  alignItems: 'stretch',
  position: 'relative',
  WebkitUserSelect: 'none',
  userSelect: 'none',
  WebkitTouchCallout: 'none',
  flexDirection: vertical ? 'column' : 'row',
  touchAction: vertical ? 'pan-x' : 'pan-y'
});

const clamp = (num, clamp, higher) => Math.min(Math.max(num, clamp), higher);

function Slider({
  children,
  index,
  onIndexChange,
  className,
  style,
  slideStyle,
  enabled,
  trail,
  vertical,
  draggedScale,
  draggedSpring,
  trailingSpring,
  trailingDelay,
  onDragStart,
  onDragEnd
}) {
  const slideStyleFunc = typeof slideStyle === 'function' ? slideStyle : () => slideStyle;
  const root = useRef();
  const {
    width,
    height
  } = useResizeObserver({
    ref: root
  });
  const axis = vertical ? 'y' : 'x';
  const size = vertical ? height : width;
  const indexRef = useRef(index);
  const restPos = useRef(0);
  const velocity = useRef(0);
  const visibleIndexes = useRef(new Set());
  const firstVisibleIndex = useRef(0);
  const lastVisibleIndex = useRef(0);

  const cb = slides => {
    slides.forEach(({
      isIntersecting,
      target
    }) => visibleIndexes.current[isIntersecting ? 'add' : 'delete'](~~target.attributes['data-index'].value));
    const visibles = Array.from(visibleIndexes.current).sort();
    firstVisibleIndex.current = visibles[0];
    lastVisibleIndex.current = visibles[visibles.length - 1];
  };

  const observer = useRef(null);
  if (!observer.current) observer.current = new IntersectionObserver(cb);
  useEffect(() => {
    Array.from(root.current.children).forEach(t => observer.current.observe(t));
  }, [children.length, root]);
  useEffect(() => () => observer.current.disconnect(), []);
  const [springs, set] = useSprings(children.length, i => {
    return {
      x: vertical ? 0 : restPos.current,
      y: vertical ? restPos.current : 0,
      s: 1,
      zIndex: 0,
      immediate: key => key === 'zIndex'
    };
  }, [children.length]);
  useEffect(() => {
    if (!width || !height) return;

    if (vertical) {
      const {
        offsetTop,
        offsetHeight
      } = root.current.children[index];
      restPos.current = Math.round(-offsetTop + (height - offsetHeight) / 2);
    } else {
      const {
        offsetLeft,
        offsetWidth
      } = root.current.children[index];
      restPos.current = Math.round(-offsetLeft + (width - offsetWidth) / 2);
    }

    if (indexRef.current === index) {
      set(() => ({
        [axis]: restPos.current,
        s: 1,
        config: key => key === axis && _extends({}, trailingSpring, {
          velocity: velocity.current
        })
      }));
    } else {
      const dir = index < indexRef.current ? -1 : 1;
      const firstToMove = dir > 0 ? firstVisibleIndex.current : lastVisibleIndex.current;
      set(i => {
        return {
          [axis]: restPos.current,
          s: 1,
          config: key => key === axis && trailingSpring,
          delay: i * dir < firstToMove * dir ? 0 : Math.abs(firstToMove - i) * trailingDelay
        };
      });
    }

    indexRef.current = index;
  }, [index, set, root, vertical, axis, height, width, trail, trailingSpring, draggedSpring, trailingDelay]);
  const bind = useDrag(({
    first,
    last,
    vxvy: [vx, vy],
    delta: [dx, dy],
    movement: [movX, movY],
    args: [pressedIndex],
    memo = springs[pressedIndex][axis].get()
  }) => {
    const v = vertical ? vy : vx;
    const dir = -Math.sign(vertical ? dy : dx);
    const mov = vertical ? movY : movX;

    if (first) {
      set(i => {
        return {
          [axis]: memo + mov,
          s: draggedScale,
          config: key => key === axis && i === pressedIndex ? draggedSpring : trailingSpring,
          zIndex: i === pressedIndex ? 10 : 0
        };
      });
      onDragStart && onDragStart(pressedIndex);
    } else if (last) {
      if (Math.abs(mov) > size / 2 || Math.abs(v) > 0.3) {
        indexRef.current = clamp(indexRef.current + (mov > 0 ? -1 : 1), 0, children.length - 1);
      }

      if (index !== indexRef.current) {
        velocity.current = v;
        requestAnimationFrame(() => onIndexChange(indexRef.current));
      } else set(() => ({
          [axis]: restPos.current,
          s: 1,
          config: key => key === axis && trailingSpring
        }));

      onDragEnd && onDragEnd(pressedIndex);
    } else {
        const firstToMove = dir > 0 ? firstVisibleIndex.current : lastVisibleIndex.current;
        set(i => {
          return {
            [axis]: mov + memo,
            delay: i * dir < firstToMove * dir || i === pressedIndex ? 0 : Math.abs(firstToMove - i) * trailingDelay,
            config: key => key === axis && i === pressedIndex ? draggedSpring : trailingSpring
          };
        });
      }

    return memo;
  }, {
    enabled,
    axis
  });
  return React.createElement("div", {
    className: className,
    style: style
  }, React.createElement("div", {
    ref: root,
    style: slidesWrapperStyle(vertical)
  }, springs.map(({
    [axis]: pos,
    s,
    zIndex
  }, i) => React.createElement(animated.div, _extends({}, bind(i), {
    key: i,
    "data-index": i,
    style: _extends({}, slideStyleFunc(i), {
      zIndex,
      [axis]: pos,
      scale: s,
      display: 'flex',
      alignItems: 'center',
      willChange: 'transform'
    })
  }), children[i]))));
}
Slider.defaultProps = {
  className: undefined,
  style: undefined,
  enabled: true,
  trail: true,
  vertical: false,
  draggedScale: 1,
  draggedSpring: {
    tension: 1200,
    friction: 40
  },
  trailingSpring: {
    mass: 1,
    tension: 280,
    friction: 60
  },
  trailingDelay: 50,
  onDragStart: undefined,
  onDragEnd: undefined
};
Slider.propTypes = {
  children: PropTypes.node.isRequired,
  index: PropTypes.number.isRequired,
  onIndexChange: PropTypes.func.isRequired,
  className: PropTypes.string,
  style: PropTypes.object,
  slideStyle: PropTypes.object,
  enabled: PropTypes.bool,
  vertical: PropTypes.bool,
  trail: PropTypes.bool,
  draggedScale: PropTypes.number,
  draggedSpring: PropTypes.object,
  trailingSpring: PropTypes.object,
  onDragStart: PropTypes.func,
  onDragEnd: PropTypes.func
};

export { Slider };
